@@ -162,7 +162,7 @@ class Config(object):
 		if not success: return response
 		
 		# disable key.
-		path = f'{H}/.ssh/config'
+		path = f'{HOME}/.ssh/config'
 		utils.__set_file_path_permission__(path, permission=755)
 		try:
 			data = utils.__load_file__(path)
@@ -471,16 +471,15 @@ class SSH(object):
 	):
 
 		# build base.
-		response = self.__build__(
+		base = self.__build__(
 			alias=alias,
 			username=username, 
 			ip=ip, 
 			port=port,
 			key_path=key_path,)
-		if response["error"] != None: return response
 
 		# execute.
-		command = response["base"]
+		command = base
 
 		# do.
 		os.system(utils.__array_to_string__(command, joiner=" "))
@@ -502,19 +501,21 @@ class SSH(object):
 	):
 
 		# build base.
-		response = self.__build__(
+		base = self.__build__(
 			alias=alias,
 			username=username, 
 			ip=ip, 
 			port=port,
 			key_path=key_path,)
-		if response["error"] != None: return response
-
+		
 		# execute.
-		command = response["base"] + command
+		command = base
 
 		# do.
 		output = utils.__execute__(command, shell=False, return_format=return_format)
+		#output = utils.__execute_script__(utils.__array_to_string__(command, joiner="\n"), shell=False, return_format=return_format)
+		
+
 		response = utils.__default_response__()
 		
 		# uncaptured error.
@@ -550,25 +551,18 @@ class SSH(object):
 		port=22,
 		key_path=None,
 	):
+		base = [
+			#f"SSH_AUTH_SOCK={agent.SSH_AUTH_SOCK}", "&&",
+			#f"SSH_AGENT_PID={agent.SSH_AGENT_PID}", "&&",
+		]
 		if alias != None:
-			response = utils.__default_response__()
-			response["base"] = ['ssh', alias]
-			return response
+			return base + ['ssh', alias]
 		else:
-			success, response = utils.__check_parameters__({
-				"username":username,
-				"ip":ip,
-				"port":port,
-				"key_path":key_path,
-			}, empty_value=None)
-			if not success: return response
-			response = utils.__default_response__()
-			base = ['ssh']
+			base += ['ssh']
 			if key_path != None: base += ['-i', key_path,]
 			if port != None: base += ['-p', port,]
 			base += [f"{username}@{ip}"]
-			response["base"] = base
-			return response
+			return base
 
 # the sshfs object class.
 class SSHFS(object):
@@ -593,7 +587,10 @@ class SSHFS(object):
 	):
 
 		# checks.
-		base = []
+		base = [
+			#f"SSH_AUTH_SOCK={agent.SSH_AUTH_SOCK}", "&&",
+			#f"SSH_AGENT_PID={agent.SSH_AGENT_PID}", "&&",
+		]
 		if alias == None:
 			success, response = utils.__check_parameters__(empty_value=None, parameters={
 				"username":username,
@@ -617,6 +614,7 @@ class SSHFS(object):
 
 		# do.
 		output = utils.__execute__(base + [f'{alias}:{server_path}', client_path], timeout=3)
+		#output = utils.__execute_script__(utils.__array_to_string__(base + [f'{alias}:{server_path}', client_path], joiner="\n"), timeout=3)
 
 		# check fails.
 		if "mount_osxfuse: mount point " in output and "is itself" in output:
@@ -725,6 +723,7 @@ class SCP(object):
 		if directory: command += ["-r"]
 		if alias == None: alias = f'{username}@{ip}'
 		output = utils.__execute__(command + [f'{alias}:{server_path}', client_path], shell=False)
+		#output = utils.__execute_script__(utils.__array_to_string__(command + [f'{alias}:{server_path}', client_path], joiner="\n"), shell=False)
 		
 		# check fails.
 		if "No such file or directory" in output:
@@ -796,6 +795,7 @@ class SCP(object):
 		if directory: command += ["-r"]
 		if alias == None: alias = f'{username}@{ip}'
 		output = utils.__execute__(command + [client_path, f'{alias}:{server_path}'], shell=False)
+		#output = utils.__execute_script__(utils.__array_to_string__(command + [client_path, f'{alias}:{server_path}'], joiner="\n"), shell=False)
 
 		# check fails.
 		if "No such file or directory" in output:
@@ -823,10 +823,14 @@ class SCP(object):
 		port=22,
 		key_path=None,
 	):
+		base = [
+			#f"SSH_AUTH_SOCK={agent.SSH_AUTH_SOCK}", "&&",
+			#f"SSH_AGENT_PID={agent.SSH_AGENT_PID}", "&&",
+		]
 		if alias != None:
-			return ['ssh', alias]
+			return base + ['ssh', alias]
 		else:
-			base = ['scp']
+			base += ['scp']
 			if key_path != None: base += ['-i', key_path,]
 			if port != None: base += ['-P', port,]
 			return base
@@ -836,8 +840,18 @@ class Agent(object):
 	def __init__(self):
 
 		# variables.
-		self.initialize()
-
+		#self.initialize()
+		
+		# set agent.
+		self.SSH_AUTH_SOCK = os.environ.get("SSH_AUTH_SOCK")
+		self.SSH_AGENT_PID = os.environ.get("SSH_AGENT_PID")
+		if self.SSH_AUTH_SOCK == None or self.SSH_AGENT_PID == None:
+			output = subprocess.check_output(["ssh-agent"]).decode()
+			self.SSH_AUTH_SOCK = output.split("SSH_AUTH_SOCK=")[1].split(";")[0]
+			self.SSH_AGENT_PID = output.split("SSH_AGENT_PID=")[1].split(";")[0]
+			os.environ["SSH_AUTH_SOCK"] = self.SSH_AUTH_SOCK
+			os.environ["SSH_AGENT_PID"] = self.SSH_AGENT_PID
+		
 		#
 	def add(self, 
 		# the keys path.
@@ -869,26 +883,27 @@ class Agent(object):
 				response["error"] = f"Key path [{path}] does not exist."
 				return response
 
-
 		# with passphrase.
 		response = utils.__default_response__()
 		if smart_card or passphrase != None:
 			self.initialize()
-			l_response = self.list()
-			if l_response["error"] != None: return l_response
-			count = len(l_response["keys"])
-			print("count 1:",count)
 			if smart_card:
 				if OS in ["osx"]:
 					copy_path = "/usr/local/lib/libykcs11_NOTALINK.dylib" # working version 1
 					os.system(f"rm -fr {copy_path}")
 					os.system(f"cp {path} {copy_path}")
 					path = copy_path
-				spawn = pexpect.spawn(f'ssh-add -s {path}')
+				file = Files.File(path="/tmp/shell.sh")
+				#file.save(f'export SSH_AUTH_SOCK={self.SSH_AUTH_SOCK}\nexport SSH_AGENT_PID={self.SSH_AGENT_PID}\nssh-add -s {path}')
+				file.save(f'ssh-add -s {path}')
+				file.file_path.permission.set(755)
+				spawn = pexpect.spawn(f'sh {file.file_path.path}')
+				file.file_path.delete(forced=True)
 			else:
 				spawn = pexpect.spawn(f'ssh-add {path}')
 
 			# send lines.
+			output = None
 			try:
 
 				# handle pincode.
@@ -908,36 +923,33 @@ class Agent(object):
 					spawn.sendline(passphrase)
 
 			except pexpect.exceptions.TIMEOUT:
-				response["error"] = f"Failed to add key [{path}] to the ssh agent #1."
-				return response
+				return r3sponse.error_response(f"Failed to add key [{path}] to the ssh agent #1.")
 			except pexpect.exceptions.EOF:
-				output = spawn.read().decode()
-				output1 = utils.__execute__(command=["ssh-add", "-l"])
-				if ("Failed to communicate" in output1 or "Error connecting to agent" in output1) or ("Error connecting to agent: No such file or directory" in output):
-					response["error"] = f"Failed to communicate with the ssh agent. You may need to logout the current system user and log back in."
-					return response
-				else:
-					response["error"] = f"Failed to add key [{path}] to the ssh agent #2."
-					return response
+				a=1
 			
+			# try output.
+			output = spawn.read().decode()
+
+			# check success.
+			if "incorrect passphrase" in output.lower():
+				return r3sponse.error_response("Provided an incorrect passphrase.")
+			elif "incorrect pin" in output.lower():
+				return r3sponse.error_response("Provided an incorrect pin code.")
+			elif "Failed to communicate" in output or "agent refused operation" in output:
+				return r3sponse.error_response("Failed to communicate with the ssh-agent. Try logging out the current system user & logging back in (or execute [$ eval `ssh-agent`]).")
+			elif "Identity added:" in output or "Card added:" in output: 
+				return r3sponse.success_response(f"Successfully added key [{path}] to the ssh agent.")
+			elif output != "": 
+				return r3sponse.error_response(f"Failed to add key [{path}] to the ssh agent, error: {output}")
+			else: 
+				return r3sponse.error_response(f"Failed to add key [{path}] to the ssh agent.")
+
 			# handle eof.
-			try:
+			"""try:
 				spawn.expect(pexpect.EOF, timeout=timeout)
 			except pexpect.ExceptionPexpect as epe:
 				response["error"] = f"Failed to add key [{path}] to the ssh agent #3."
-				return response
-
-			# check success.
-			l_response = self.list()
-			if l_response["error"] != None: return l_response
-			print("count 2:",len(l_response["keys"]))
-			if count != len(l_response["keys"]):
-				response["success"] = True
-				response["message"] = f"Successfully added key [{path}] to the ssh agent."
-				return response
-			else: 
-				response["error"] = f"Failed to add key [{path}] to the ssh agent."
-				return response
+				return response"""
 
 		# without pass.
 		else:
@@ -945,15 +957,11 @@ class Agent(object):
 
 			# check success.
 			if "Failed to communicate" in output:
-				response["error"] = "Failed to communicate with the ssh-agent."
-				return response
+				return r3sponse.error_response("Failed to communicate with the ssh-agent.")
 			elif "Identity added:" in output: 
-				response["success"] = True
-				response["message"] = f"Successfully added key [{path}] to the ssh agent."
-				return response
+				return r3sponse.success_response(f"Successfully added key [{path}] to the ssh agent.")
 			else: 
-				response["error"] = f"Failed to add key [{path}] to the ssh agent."
-				return response
+				return r3sponse.error_response(f"Failed to add key [{path}] to the ssh agent.")
 
 		#
 	def delete(self):
